@@ -1,8 +1,8 @@
 /**************************************************************************
  *   global.c  --  This file is part of GNU nano.                         *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2020 Free Software Foundation, Inc.    *
- *   Copyright (C) 2014-2020 Benno Schulenberg                            *
+ *   Copyright (C) 1999-2011, 2013-2022 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2014-2021 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -49,7 +49,8 @@ bool we_are_running = FALSE;
 		/* Becomes TRUE as soon as all options and files have been read. */
 bool more_than_one = FALSE;
 		/* Whether more than one buffer is or has been open. */
-
+bool report_size = TRUE;
+		/* Whether to show the number of lines when the minibar is used. */
 bool ran_a_tool = FALSE;
 		/* Whether a tool has been run at the Execute-Command prompt. */
 
@@ -117,12 +118,12 @@ size_t wrap_at = 0;
 WINDOW *topwin = NULL;
 		/* The top portion of the screen, showing the version number of nano,
 		 * the name of the file, and whether the buffer was modified. */
-WINDOW *edit = NULL;
+WINDOW *midwin = NULL;
 		/* The middle portion of the screen: the edit window, showing the
 		 * contents of the current buffer, the file we are editing. */
-WINDOW *bottomwin = NULL;
-		/* The bottom portion of the screen, where we display statusbar
-		 * messages, the status-bar prompt, and a list of shortcuts. */
+WINDOW *footwin = NULL;
+		/* The bottom portion of the screen, where status-bar messages,
+		 * the status-bar prompt, and a list of shortcuts are shown. */
 int editwinrows = 0;
 		/* How many rows does the edit window take up? */
 int editwincols = -1;
@@ -215,15 +216,16 @@ funcstruct *exitfunc;
 
 linestruct *search_history = NULL;
 		/* The current item in the list of strings that were searched for. */
-linestruct *execute_history = NULL;
-		/* The current item in the list of commands that were run with ^R ^X. */
 linestruct *replace_history = NULL;
 		/* The current item in the list of replace strings. */
+linestruct *execute_history = NULL;
+		/* The current item in the list of commands that were run with ^T. */
+
 #ifdef ENABLE_HISTORIES
 linestruct *searchtop = NULL;
 		/* The oldest item in the list of search strings. */
 linestruct *searchbot = NULL;
-		/* The newest item in the list of search strings. */
+		/* The empty item at the end of the list of search strings. */
 
 linestruct *replacetop = NULL;
 linestruct *replacebot = NULL;
@@ -279,78 +281,36 @@ size_t light_to_col = 0;
 #endif
 
 /* Empty functions, for the most part corresponding to toggles. */
-void case_sens_void(void)
-{
-}
-void regexp_void(void)
-{
-}
-void backwards_void(void)
-{
-}
-void flip_replace(void)
-{
-}
-void flip_goto(void)
-{
-}
+void case_sens_void(void)  {;}
+void regexp_void(void)  {;}
+void backwards_void(void)  {;}
+#ifdef ENABLE_HISTORIES
+void get_older_item(void)  {;}
+void get_newer_item(void)  {;}
+#endif
+void flip_replace(void)  {;}
+void flip_goto(void)  {;}
 #ifdef ENABLE_BROWSER
-void to_files(void)
-{
-}
-void to_first_file(void)
-{
-}
-void to_last_file(void)
-{
-}
-void goto_dir(void)
-{
-}
+void to_files(void)  {;}
+void goto_dir(void)  {;}
 #endif
 #ifndef NANO_TINY
-void do_nothing(void)
-{
-}
-void do_toggle_void(void)
-{
-}
-void dos_format_void(void)
-{
-}
-void mac_format_void(void)
-{
-}
-void append_void(void)
-{
-}
-void prepend_void(void)
-{
-}
-void backup_file_void(void)
-{
-}
-void flip_execute(void)
-{
-}
-void flip_pipe(void)
-{
-}
-void flip_convert(void)
-{
-}
+void do_nothing(void)  {;}
+void do_toggle(void)  {;}
+void dos_format(void)  {;}
+void mac_format(void)  {;}
+void append_it(void)  {;}
+void prepend_it(void)  {;}
+void back_it_up(void)  {;}
+void flip_execute(void)  {;}
+void flip_pipe(void)  {;}
+void flip_convert(void)  {;}
 #endif
 #ifdef ENABLE_MULTIBUFFER
-void flip_newbuffer(void)
-{
-}
+void flip_newbuffer(void)  {;}
 #endif
-void discard_buffer(void)
-{
-}
-void do_cancel(void)
-{
-}
+void discard_buffer(void)  {;}
+void do_cancel(void)  {;}
 
 /* Add a function to the linked list of functions. */
 void add_to_funcs(void (*func)(void), int menus, const char *desc,
@@ -381,7 +341,7 @@ int keycode_from_string(const char *keystring)
 {
 	if (keystring[0] == '^') {
 		if (keystring[2] == '\0') {
-			if (keystring[1] == '/')
+			if (keystring[1] == '/' || keystring[1] == '-')
 				return 31;
 			if (keystring[1] <= '_')
 				return keystring[1] - 64;
@@ -477,7 +437,8 @@ size_t shown_entries_for(int menu)
 	}
 
 	/* When --saveonexit is not used, widen the grid of the WriteOut menu. */
-	if (menu == MWRITEFILE && first_sc_for(menu, discard_buffer) == NULL)
+	if (menu == MWRITEFILE && item == NULL &&
+						first_sc_for(menu, discard_buffer) == NULL)
 		count--;
 
 	return count;
@@ -529,6 +490,7 @@ functionptrtype interpret(int *keycode)
 			return do_findnext;
 
 		switch (tolower(*keycode)) {
+			case 'b':
 			case '-':
 				return do_page_up;
 			case ' ':
@@ -555,6 +517,14 @@ functionptrtype interpret(int *keycode)
 }
 #endif /* ENABLE_BROWSER || ENABLE_HELP */
 
+#if defined(NANO_TINY) && defined(ENABLE_LINENUMBERS)
+/* Allow toggling line numbers also in the tiny version. */
+void toggle_numbers(void)
+{
+	TOGGLE(LINE_NUMBERS);
+}
+#endif
+
 /* These two tags are used elsewhere too, so they are global. */
 /* TRANSLATORS: Try to keep the next two strings at most 10 characters. */
 const char *exit_tag = N_("Exit");
@@ -632,7 +602,7 @@ void shortcut_init(void)
 	const char *nextfile_gist = N_("Switch to the next file buffer");
 #endif
 	const char *verbatim_gist = N_("Insert the next keystroke verbatim");
-	const char *tab_gist = N_("Insert a tab at the cursor position");
+	const char *tab_gist = N_("Insert a tab at the cursor position (or indent marked lines)");
 	const char *enter_gist = N_("Insert a newline at the cursor position");
 	const char *delete_gist = N_("Delete the character under the cursor");
 	const char *backspace_gist =
@@ -651,10 +621,10 @@ void shortcut_init(void)
 #endif
 #ifndef NANO_TINY
 	const char *wordcount_gist =
-		N_("Count the number of words, lines, and characters");
+		N_("Count the number of lines, words, and characters");
+	const char *suspend_gist = N_("Suspend the editor (return to the shell)");
 #endif
 	const char *refresh_gist = N_("Refresh (redraw) the current screen");
-	const char *suspend_gist = N_("Suspend the editor (if suspension is enabled)");
 #ifdef ENABLE_WORDCOMPLETION
 	const char *completion_gist = N_("Try and complete the current word");
 #endif
@@ -728,10 +698,11 @@ void shortcut_init(void)
 #endif
 
 	/* Start populating the different menus with functions. */
-
+#ifdef ENABLE_HELP
 	add_to_funcs(do_help, (MMOST | MBROWSER) & ~MFINDINHELP,
 		/* TRANSLATORS: Try to keep the next thirteen strings at most 10 characters. */
 		N_("Help"), WITHORSANS(help_gist), TOGETHER, VIEW);
+#endif
 
 	add_to_funcs(do_cancel, ((MMOST & ~MMAIN) | MYESNO),
 		N_("Cancel"), WITHORSANS(cancel_gist), BLANKAFTER, VIEW);
@@ -746,7 +717,16 @@ void shortcut_init(void)
 		close_tag, WITHORSANS(exitbrowser_gist), TOGETHER, VIEW);
 #endif
 
-	add_to_funcs(do_writeout_void, MMAIN,
+#ifndef ENABLE_HELP
+	add_to_funcs(full_refresh, MMAIN|MREPLACE, "Refresh", "x", 0, VIEW);
+#ifndef NANO_TINY
+	add_to_funcs(full_refresh, MINSERTFILE|MEXECUTE, "Refresh", "x", 0, VIEW);
+#endif
+	add_to_funcs(flip_goto, MWHEREIS, "Go To Line", "x", 0, VIEW);
+	add_to_funcs(flip_goto, MGOTOLINE, "Go To Text", "x", 0, VIEW);
+#endif
+
+	add_to_funcs(do_writeout, MMAIN,
 		N_("Write Out"), WITHORSANS(writeout_gist), TOGETHER, NOVIEW);
 
 #ifdef ENABLE_JUSTIFY
@@ -754,14 +734,14 @@ void shortcut_init(void)
 	 * otherwise, show Insert anyway, to keep the help items paired. */
 	if (!ISSET(RESTRICTED))
 #endif
-		add_to_funcs(do_insertfile_void, MMAIN,
+		add_to_funcs(do_insertfile, MMAIN,
 				N_("Read File"), WITHORSANS(readfile_gist), BLANKAFTER,
 				/* We allow inserting files in view mode if multibuffer mode
 				 * is available, so that the user can view multiple files. */
 				CAN_OPEN_OTHER_BUFFER);
 #ifdef ENABLE_JUSTIFY
 	else
-		add_to_funcs(do_justify_void, MMAIN,
+		add_to_funcs(do_justify, MMAIN,
 				N_("Justify"), WITHORSANS(justify_gist), BLANKAFTER, NOVIEW);
 #endif
 
@@ -778,6 +758,16 @@ void shortcut_init(void)
 	add_to_funcs(do_replace, MMAIN,
 		N_("Replace"), WITHORSANS(replace_gist), TOGETHER, NOVIEW);
 
+#ifdef NANO_TINY
+	add_to_funcs(do_search_backward, MHELP,
+		N_("Where Was"), WITHORSANS(wherewas_gist), TOGETHER, VIEW);
+
+	add_to_funcs(do_findprevious, MMAIN|MHELP,
+		N_("Previous"), WITHORSANS(findprev_gist), TOGETHER, VIEW);
+	add_to_funcs(do_findnext, MMAIN|MHELP,
+		N_("Next"), WITHORSANS(findnext_gist), BLANKAFTER, VIEW);
+#endif
+
 	add_to_funcs(cut_text, MMAIN,
 		N_("Cut"), WITHORSANS(cut_gist), TOGETHER, NOVIEW);
 
@@ -790,18 +780,19 @@ void shortcut_init(void)
 				N_("Execute"), WITHORSANS(execute_gist), TOGETHER, NOVIEW);
 #endif
 #ifdef ENABLE_JUSTIFY
-		add_to_funcs(do_justify_void, MMAIN,
+		add_to_funcs(do_justify, MMAIN,
 				N_("Justify"), WITHORSANS(justify_gist), BLANKAFTER, NOVIEW);
 #endif
 	}
 
 	add_to_funcs(report_cursor_position, MMAIN,
+		/* TRANSLATORS: This refers to the position of the cursor. */
 		N_("Location"), WITHORSANS(cursorpos_gist), TOGETHER, VIEW);
 
 #if defined(NANO_TINY) || defined(ENABLE_JUSTIFY)
 	/* Conditionally placing this one here or further on, to keep the
 	 * help items nicely paired in most conditions. */
-	add_to_funcs(do_gotolinecolumn_void, MMAIN,
+	add_to_funcs(do_gotolinecolumn, MMAIN,
 		N_("Go To Line"), WITHORSANS(gotoline_gist), BLANKAFTER, VIEW);
 #endif
 
@@ -815,7 +806,7 @@ void shortcut_init(void)
 	add_to_funcs(do_mark, MMAIN,
 		N_("Set Mark"), WITHORSANS(mark_gist), TOGETHER, VIEW);
 	add_to_funcs(copy_text, MMAIN,
-		N_("Copy"), WITHORSANS(copy_gist), BLANKAFTER, NOVIEW);
+		N_("Copy"), WITHORSANS(copy_gist), BLANKAFTER, VIEW);
 #endif
 
 	add_to_funcs(case_sens_void, MWHEREIS|MREPLACE,
@@ -827,51 +818,59 @@ void shortcut_init(void)
 
 	add_to_funcs(flip_replace, MWHEREIS,
 		N_("Replace"), WITHORSANS(replace_gist), BLANKAFTER, VIEW);
-
 	add_to_funcs(flip_replace, MREPLACE,
 		N_("No Replace"), WITHORSANS(whereis_gist), BLANKAFTER, VIEW);
 
 #ifdef ENABLE_HISTORIES
-	add_to_funcs(get_history_older_void, MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE,
+	add_to_funcs(get_older_item, MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE,
 		N_("Older"), WITHORSANS(older_gist), TOGETHER, VIEW);
-	add_to_funcs(get_history_newer_void, MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE,
+	add_to_funcs(get_newer_item, MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE,
 		N_("Newer"), WITHORSANS(newer_gist), BLANKAFTER, VIEW);
 #endif
 
+#ifdef ENABLE_HELP
 	add_to_funcs(flip_goto, MWHEREIS,
 		N_("Go To Line"), WITHORSANS(gotoline_gist), BLANKAFTER, VIEW);
+#endif
 
 #ifdef ENABLE_BROWSER
 	add_to_funcs(goto_dir, MBROWSER,
-		/* TRANSLATORS: Try to keep the next seven strings at most 10 characters. */
+		/* TRANSLATORS: Try to keep the next four strings at most 10 characters. */
 		N_("Go To Dir"), WITHORSANS(gotodir_gist), TOGETHER, VIEW);
-
+#ifdef ENABLE_HELP
 	add_to_funcs(full_refresh, MBROWSER,
 		N_("Refresh"), WITHORSANS(browserrefresh_gist), BLANKAFTER, VIEW);
-
+#endif
 	add_to_funcs(do_search_forward, MBROWSER,
 		N_("Where Is"), WITHORSANS(browserwhereis_gist), TOGETHER, VIEW);
 	add_to_funcs(do_search_backward, MBROWSER,
 		N_("Where Was"), WITHORSANS(browserwherewas_gist), TOGETHER, VIEW);
+
+	add_to_funcs(do_findprevious, MBROWSER,
+		N_("Previous"), WITHORSANS(findprev_gist), TOGETHER, VIEW);
+	add_to_funcs(do_findnext, MBROWSER,
+		N_("Next"), WITHORSANS(findnext_gist), BLANKAFTER, VIEW);
 #endif
 
-#ifndef NANO_TINY
+#ifdef NANO_TINY
+	add_to_funcs(to_prev_word, MMAIN,
+		"Prev Word", WITHORSANS(prevword_gist), TOGETHER, VIEW);
+	add_to_funcs(to_next_word, MMAIN,
+		"Next Word", WITHORSANS(nextword_gist), BLANKAFTER, VIEW);
+#else
 	add_to_funcs(do_find_bracket, MMAIN,
 		N_("To Bracket"), WITHORSANS(bracket_gist), BLANKAFTER, VIEW);
 
 	add_to_funcs(do_search_backward, MMAIN|MHELP,
 		/* TRANSLATORS: This starts a backward search. */
 		N_("Where Was"), WITHORSANS(wherewas_gist), TOGETHER, VIEW);
-#else
-	add_to_funcs(do_search_backward, MHELP,
-		N_("Where Was"), WITHORSANS(wherewas_gist), TOGETHER, VIEW);
-#endif
 
-	add_to_funcs(do_findprevious, MMAIN|MBROWSER|MHELP,
+	add_to_funcs(do_findprevious, MMAIN|MHELP,
 		/* TRANSLATORS: This refers to searching the preceding occurrence. */
 		N_("Previous"), WITHORSANS(findprev_gist), TOGETHER, VIEW);
-	add_to_funcs(do_findnext, MMAIN|MBROWSER|MHELP,
+	add_to_funcs(do_findnext, MMAIN|MHELP,
 		N_("Next"), WITHORSANS(findnext_gist), BLANKAFTER, VIEW);
+#endif
 
 	add_to_funcs(do_left, MMAIN,
 		/* TRANSLATORS: This means move the cursor one character back. */
@@ -885,11 +884,13 @@ void shortcut_init(void)
 		N_("Forward"), WITHORSANS(forwardfile_gist), TOGETHER, VIEW);
 #endif
 
+#ifndef NANO_TINY
 	add_to_funcs(to_prev_word, MMAIN,
 		/* TRANSLATORS: Try to keep the next ten strings at most 12 characters. */
 		N_("Prev Word"), WITHORSANS(prevword_gist), TOGETHER, VIEW);
 	add_to_funcs(to_next_word, MMAIN,
 		N_("Next Word"), WITHORSANS(nextword_gist), TOGETHER, VIEW);
+#endif
 
 	add_to_funcs(do_home, MMAIN,
 		N_("Home"), WITHORSANS(home_gist), TOGETHER, VIEW);
@@ -913,7 +914,7 @@ void shortcut_init(void)
 		N_("Next Block"), WITHORSANS(nextblock_gist), TOGETHER, VIEW);
 #ifdef ENABLE_JUSTIFY
 	add_to_funcs(to_para_begin, MMAIN|MGOTOLINE,
-		/* TRANSLATORS: Try to keep these two strings at most 15 characters. */
+		/* TRANSLATORS: Try to keep these two strings at most 16 characters. */
 		N_("Begin of Paragr."), WITHORSANS(parabegin_gist), TOGETHER, VIEW);
 	add_to_funcs(to_para_end, MMAIN|MGOTOLINE,
 		N_("End of Paragraph"), WITHORSANS(paraend_gist), BLANKAFTER, VIEW);
@@ -938,7 +939,7 @@ void shortcut_init(void)
 #endif
 
 #if !defined(NANO_TINY) && !defined(ENABLE_JUSTIFY)
-	add_to_funcs(do_gotolinecolumn_void, MMAIN,
+	add_to_funcs(do_gotolinecolumn, MMAIN,
 		N_("Go To Line"), WITHORSANS(gotoline_gist), BLANKAFTER, VIEW);
 #endif
 
@@ -975,18 +976,21 @@ void shortcut_init(void)
 #endif
 
 #ifndef NANO_TINY
-	add_to_funcs(do_wordlinechar_count, MMAIN,
+	add_to_funcs(count_lines_words_and_characters, MMAIN,
 		N_("Word Count"), WITHORSANS(wordcount_gist), TOGETHER, VIEW);
 #endif
 
 	add_to_funcs(do_verbatim_input, MMAIN,
 		N_("Verbatim"), WITHORSANS(verbatim_gist), BLANKAFTER, NOVIEW);
 
+#ifndef NANO_TINY
+	add_to_funcs(do_suspend, MMAIN,
+		N_("Suspend"), WITHORSANS(suspend_gist), TOGETHER, VIEW);
+#endif
+#ifdef ENABLE_HELP
 	add_to_funcs(full_refresh, MMAIN,
-		N_("Refresh"), WITHORSANS(refresh_gist), TOGETHER, VIEW);
-
-	add_to_funcs(do_suspend_void, MMAIN,
-		N_("Suspend"), WITHORSANS(suspend_gist), BLANKAFTER, VIEW);
+		N_("Refresh"), WITHORSANS(refresh_gist), BLANKAFTER, VIEW);
+#endif
 
 #ifndef NANO_TINY
 	add_to_funcs(do_indent, MMAIN,
@@ -1002,6 +1006,7 @@ void shortcut_init(void)
 	add_to_funcs(complete_a_word, MMAIN,
 		N_("Complete"), WITHORSANS(completion_gist), BLANKAFTER, NOVIEW);
 #endif
+
 #ifndef NANO_TINY
 	add_to_funcs(record_macro, MMAIN,
 		N_("Record"), WITHORSANS(recordmacro_gist), TOGETHER, VIEW);
@@ -1016,7 +1021,7 @@ void shortcut_init(void)
 		N_("Down to anchor"), WITHORSANS(nextanchor_gist), BLANKAFTER, VIEW);
 
 	add_to_funcs(zap_text, MMAIN,
-		/* TRANSLATORS: This *deletes* a line or marked region. */
+		/* TRANSLATORS: This refers to deleting a line or marked region. */
 		N_("Zap"), WITHORSANS(zap_gist), BLANKAFTER, NOVIEW);
 
 	if (!ISSET(RESTRICTED)) {
@@ -1032,10 +1037,12 @@ void shortcut_init(void)
 #endif
 	}
 #endif /* !NANO_TINY */
+
 #ifdef NANO_TINY
 	add_to_funcs(do_search_backward, MMAIN,
 		N_("Where Was"), WITHORSANS(wherewas_gist), BLANKAFTER, VIEW);
 #endif
+
 #if !defined(NANO_TINY) || defined(ENABLE_HELP)
 	add_to_funcs(do_center, MMAIN,
 		N_("Center"), WITHORSANS(center_gist), BLANKAFTER, VIEW);
@@ -1070,13 +1077,15 @@ void shortcut_init(void)
 			N_("Formatter"), WITHORSANS(formatter_gist), BLANKAFTER, NOVIEW);
 #endif
 
+#ifdef ENABLE_HELP
 	add_to_funcs(flip_goto, MGOTOLINE,
 		N_("Go To Text"), WITHORSANS(whereis_gist), BLANKAFTER, VIEW);
+#endif
 
 #ifndef NANO_TINY
-	add_to_funcs(dos_format_void, MWRITEFILE,
+	add_to_funcs(dos_format, MWRITEFILE,
 		N_("DOS Format"), WITHORSANS(dos_gist), TOGETHER, NOVIEW);
-	add_to_funcs(mac_format_void, MWRITEFILE,
+	add_to_funcs(mac_format, MWRITEFILE,
 		N_("Mac Format"), WITHORSANS(mac_gist), TOGETHER, NOVIEW);
 
 	/* If we're using restricted mode, the Append, Prepend, and Backup toggles
@@ -1084,12 +1093,12 @@ void shortcut_init(void)
 	 * reduplicating the current file, and the third is not allowed as it
 	 * would write to a file not specified on the command line. */
 	if (!ISSET(RESTRICTED)) {
-		add_to_funcs(append_void, MWRITEFILE,
+		add_to_funcs(append_it, MWRITEFILE,
 			N_("Append"), WITHORSANS(append_gist), TOGETHER, NOVIEW);
-		add_to_funcs(prepend_void, MWRITEFILE,
+		add_to_funcs(prepend_it, MWRITEFILE,
 			N_("Prepend"), WITHORSANS(prepend_gist), TOGETHER, NOVIEW);
 
-		add_to_funcs(backup_file_void, MWRITEFILE,
+		add_to_funcs(back_it_up, MWRITEFILE,
 			N_("Backup File"), WITHORSANS(backup_gist), BLANKAFTER, NOVIEW);
 	}
 
@@ -1104,10 +1113,11 @@ void shortcut_init(void)
 		add_to_funcs(cut_till_eof, MEXECUTE,
 			N_("Cut Till End"), WITHORSANS(cuttilleof_gist), BLANKAFTER, NOVIEW);
 
-		add_to_funcs(do_suspend_void, MEXECUTE,
+		add_to_funcs(do_suspend, MEXECUTE,
 			N_("Suspend"), WITHORSANS(suspend_gist), BLANKAFTER, VIEW);
 	}
 #endif /* !NANO_TINY */
+
 #ifdef ENABLE_BROWSER
 	/* The file browser is only available when not in restricted mode. */
 	if (!ISSET(RESTRICTED))
@@ -1124,6 +1134,7 @@ void shortcut_init(void)
 		N_("First File"), WITHORSANS(firstfile_gist), TOGETHER, VIEW);
 	add_to_funcs(to_last_file, MBROWSER|MWHEREISFILE,
 		N_("Last File"), WITHORSANS(lastfile_gist), BLANKAFTER, VIEW);
+
 #ifndef NANO_TINY
 	add_to_funcs(to_prev_word, MBROWSER,
 		N_("Left Column"), WITHORSANS(browserlefthand_gist), TOGETHER, VIEW);
@@ -1147,6 +1158,12 @@ void shortcut_init(void)
 		N_("Next Linter message"), WITHORSANS(nextlint_gist), TOGETHER, VIEW);
 #endif
 
+#ifdef __linux__
+#define SLASH_OR_DASH  (on_a_vt) ? "^-" : "^/"
+#else
+#define SLASH_OR_DASH  "^/"
+#endif
+
 	/* Link key combos to functions in certain menus. */
 	add_to_sclist(MMOST|MBROWSER, "^M", '\r', do_enter, 0);
 	add_to_sclist(MMOST|MBROWSER, "Enter", KEY_ENTER, do_enter, 0);
@@ -1161,25 +1178,32 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN|MBROWSER|MHELP, "^X", 0, do_exit, 0);
 	if (!ISSET(PRESERVE))
 		add_to_sclist(MMAIN, "^S", 0, do_savefile, 0);
-	add_to_sclist(MMAIN, "^O", 0, do_writeout_void, 0);
-	add_to_sclist(MMAIN, "^R", 0, do_insertfile_void, 0);
-	add_to_sclist(MMAIN, "Ins", KEY_IC, do_insertfile_void, 0);
+	add_to_sclist(MMAIN, "^O", 0, do_writeout, 0);
+	add_to_sclist(MMAIN, "^R", 0, do_insertfile, 0);
+	add_to_sclist(MMAIN, "Ins", KEY_IC, do_insertfile, 0);
 	if (!ISSET(PRESERVE))
 		add_to_sclist(MMAIN|MBROWSER|MHELP, "^Q", 0, do_search_backward, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP, "^W", 0, do_search_forward, 0);
 	add_to_sclist(MMAIN, "^\\", 0, do_replace, 0);
 	add_to_sclist(MMAIN, "M-R", 0, do_replace, 0);
 	add_to_sclist(MMOST, "^K", 0, cut_text, 0);
+#ifdef NANO_TINY
+	add_to_sclist(MMAIN, "^U", 0, paste_text, 0);
+#else
 	add_to_sclist(MMOST, "^U", 0, paste_text, 0);
-#ifndef NANO_TINY
 	add_to_sclist(MMAIN, "^T", 0, do_execute, 0);
 #endif
 #ifdef ENABLE_JUSTIFY
-	add_to_sclist(MMAIN, "^J", '\n', do_justify_void, 0);
+	add_to_sclist(MMAIN, "^J", '\n', do_justify, 0);
 #endif
 #ifdef ENABLE_SPELLER
-	add_to_sclist(MEXECUTE, "^S", 0, do_spell, 0);
+#ifndef NANO_TINY
+	if (!ISSET(PRESERVE))
+		add_to_sclist(MEXECUTE, "^S", 0, do_spell, 0);
 	add_to_sclist(MEXECUTE, "^T", 0, do_spell, 0);
+#else
+	add_to_sclist(MMAIN, "^T", 0, do_spell, 0);
+#endif
 #endif
 #ifdef ENABLE_COLOR
 	add_to_sclist(MMAIN, "M-B", 0, do_linter, 0);
@@ -1188,8 +1212,9 @@ void shortcut_init(void)
 	add_to_sclist(MEXECUTE, "^O", 0, do_formatter, 0);
 #endif
 	add_to_sclist(MMAIN, "^C", 0, report_cursor_position, 0);
-	add_to_sclist(MMAIN, "^_", 0, do_gotolinecolumn_void, 0);
-	add_to_sclist(MMAIN, "M-G", 0, do_gotolinecolumn_void, 0);
+	add_to_sclist(MMAIN, SLASH_OR_DASH, 0, do_gotolinecolumn, 0);
+	add_to_sclist(MMAIN, "M-G", 0, do_gotolinecolumn, 0);
+	add_to_sclist(MMAIN, "^_", 0, do_gotolinecolumn, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "^Y", 0, do_page_up, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "PgUp", KEY_PPAGE, do_page_up, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "^V", 0, do_page_down, 0);
@@ -1203,15 +1228,24 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN|MHELP, "^End", CONTROL_END, to_last_line, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP, "M-W", 0, do_findnext, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP, "M-Q", 0, do_findprevious, 0);
-#ifndef NANO_TINY
+#ifdef NANO_TINY
+#ifdef ENABLE_LINENUMBERS
+	add_to_sclist(MMAIN, "M-N", 0, toggle_numbers, 0);
+#else
+	add_to_sclist(MMAIN, "M-B", 0, to_prev_word, 0);
+	add_to_sclist(MMAIN, "M-N", 0, to_next_word, 0);
+#endif
+	add_to_sclist(MMAIN, "M-D", 0, to_prev_word, 0);
+	add_to_sclist(MMAIN, "M-F", 0, to_next_word, 0);
+#else
 	add_to_sclist(MMAIN, "M-]", 0, do_find_bracket, 0);
 	add_to_sclist(MMAIN, "M-A", 0, do_mark, 0);
 	add_to_sclist(MMAIN, "^6", 0, do_mark, 0);
 	add_to_sclist(MMAIN, "^^", 0, do_mark, 0);
-	add_to_sclist(MMAIN, "M-6", 0, copy_text, 0);
-	add_to_sclist(MMAIN, "M-^", 0, copy_text, 0);
+	add_to_sclist(MMOST, "M-6", 0, copy_text, 0);
+	add_to_sclist(MMOST, "M-^", 0, copy_text, 0);
 	add_to_sclist(MMAIN, "M-}", 0, do_indent, 0);
-	add_to_sclist(MMAIN, "Tab", INDENT_KEY, do_indent, 0);
+	add_to_sclist(MMAIN, "", INDENT_KEY, do_indent, 0);
 	add_to_sclist(MMAIN, "M-{", 0, do_unindent, 0);
 	add_to_sclist(MMAIN, "Sh-Tab", SHIFT_TAB, do_unindent, 0);
 	add_to_sclist(MMAIN, "M-:", 0, record_macro, 0);
@@ -1236,14 +1270,14 @@ void shortcut_init(void)
 	add_to_sclist(MMOST|MBROWSER, "^F", 0, do_right, 0);
 #ifdef ENABLE_UTF8
 	if (using_utf8()) {
-		add_to_sclist(MMOST|MBROWSER|MHELP, "\xE2\x97\x80", KEY_LEFT, do_left, 0);
-		add_to_sclist(MMOST|MBROWSER|MHELP, "\xE2\x96\xb6", KEY_RIGHT, do_right, 0);
-		add_to_sclist(MSOME, "^\xE2\x97\x80", CONTROL_LEFT, to_prev_word, 0);
-		add_to_sclist(MSOME, "^\xE2\x96\xb6", CONTROL_RIGHT, to_next_word, 0);
+		add_to_sclist(MMOST|MBROWSER|MHELP, "\xE2\x97\x82", KEY_LEFT, do_left, 0);
+		add_to_sclist(MMOST|MBROWSER|MHELP, "\xE2\x96\xb8", KEY_RIGHT, do_right, 0);
+		add_to_sclist(MSOME, "^\xE2\x97\x82", CONTROL_LEFT, to_prev_word, 0);
+		add_to_sclist(MSOME, "^\xE2\x96\xb8", CONTROL_RIGHT, to_next_word, 0);
 #if !defined(NANO_TINY) && defined(ENABLE_MULTIBUFFER)
 		if (!on_a_vt) {
-			add_to_sclist(MMAIN, "M-\xE2\x97\x80", ALT_LEFT, switch_to_prev_buffer, 0);
-			add_to_sclist(MMAIN, "M-\xE2\x96\xb6", ALT_RIGHT, switch_to_next_buffer, 0);
+			add_to_sclist(MMAIN, "M-\xE2\x97\x82", ALT_LEFT, switch_to_prev_buffer, 0);
+			add_to_sclist(MMAIN, "M-\xE2\x96\xb8", ALT_RIGHT, switch_to_next_buffer, 0);
 		}
 #endif
 	} else
@@ -1260,12 +1294,6 @@ void shortcut_init(void)
 		}
 #endif
 	}
-#ifdef NANO_TINY
-	add_to_sclist(MMAIN, "M-B", 0, to_prev_word, 0);
-	add_to_sclist(MMAIN, "M-D", 0, to_prev_word, 0);
-	add_to_sclist(MMAIN, "M-F", 0, to_next_word, 0);
-	add_to_sclist(MMAIN, "M-N", 0, to_next_word, 0);
-#endif
 	add_to_sclist(MMOST, "M-Space", 0, to_prev_word, 0);
 	add_to_sclist(MMOST, "^Space", 0, to_next_word, 0);
 	add_to_sclist(MMOST, "^A", 0, do_home, 0);
@@ -1276,10 +1304,10 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN|MBROWSER|MHELP, "^N", 0, do_down, 0);
 #ifdef ENABLE_UTF8
 	if (using_utf8()) {
-		add_to_sclist(MMAIN|MBROWSER|MHELP, "\xE2\x96\xb2", KEY_UP, do_up, 0);
-		add_to_sclist(MMAIN|MBROWSER|MHELP, "\xE2\x96\xbc", KEY_DOWN, do_down, 0);
-		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^\xE2\x96\xb2", CONTROL_UP, to_prev_block, 0);
-		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^\xE2\x96\xbc", CONTROL_DOWN, to_next_block, 0);
+		add_to_sclist(MMAIN|MBROWSER|MHELP, "\xE2\x96\xb4", KEY_UP, do_up, 0);
+		add_to_sclist(MMAIN|MBROWSER|MHELP, "\xE2\x96\xbe", KEY_DOWN, do_down, 0);
+		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^\xE2\x96\xb4", CONTROL_UP, to_prev_block, 0);
+		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^\xE2\x96\xbe", CONTROL_DOWN, to_next_block, 0);
 	} else
 #endif
 	{
@@ -1299,8 +1327,8 @@ void shortcut_init(void)
 #ifndef NANO_TINY
 #ifdef ENABLE_UTF8
 	if (using_utf8()) {
-		add_to_sclist(MMAIN|MHELP, "M-\xE2\x96\xb2", ALT_UP, do_scroll_up, 0);
-		add_to_sclist(MMAIN|MHELP, "M-\xE2\x96\xbc", ALT_DOWN, do_scroll_down, 0);
+		add_to_sclist(MMAIN|MHELP, "M-\xE2\x96\xb4", ALT_UP, do_scroll_up, 0);
+		add_to_sclist(MMAIN|MHELP, "M-\xE2\x96\xbe", ALT_DOWN, do_scroll_down, 0);
 	} else
 #endif
 	{
@@ -1324,7 +1352,11 @@ void shortcut_init(void)
 #ifndef NANO_TINY
 	add_to_sclist(MMAIN, "M-T", 0, cut_till_eof, 0);
 	add_to_sclist(MEXECUTE, "^V", 0, cut_till_eof, 0);
-	add_to_sclist(MMAIN, "M-D", 0, do_wordlinechar_count, 0);
+	add_to_sclist(MEXECUTE, "^Z", 0, do_suspend, 0);
+	add_to_sclist(MMAIN, "^Z", 0, suggest_ctrlT_ctrlZ, 0);
+	add_to_sclist(MMAIN, "M-D", 0, count_lines_words_and_characters, 0);
+#else
+	add_to_sclist(MMAIN, "M-H", 0, do_help, 0);
 #endif
 #ifdef ENABLE_JUSTIFY
 	add_to_sclist(MMAIN, "M-J", 0, do_full_justify, 0);
@@ -1337,37 +1369,34 @@ void shortcut_init(void)
 		add_to_sclist(MMOST|MBROWSER|MHELP|MYESNO, "^L", 0, full_refresh, 0);
 	else
 		add_to_sclist(MMOST|MBROWSER|MYESNO, "^L", 0, full_refresh, 0);
-	add_to_sclist(MMAIN|MEXECUTE, "^Z", 0, do_suspend_void, 0);
 
 #ifndef NANO_TINY
 	/* Group of "Appearance" toggles. */
-	add_to_sclist((MMOST|MBROWSER|MYESNO) & ~MFINDINHELP, "M-X", 0, do_toggle_void, NO_HELP);
-	add_to_sclist(MMAIN, "M-C", 0, do_toggle_void, CONSTANT_SHOW);
-	add_to_sclist(MMAIN, "M-S", 0, do_toggle_void, SOFTWRAP);
-	add_to_sclist(MMAIN, "M-$", 0, do_toggle_void, SOFTWRAP);
+	add_to_sclist(MMAIN, "M-Z", 0, do_toggle, ZERO);
+	add_to_sclist((MMOST|MBROWSER|MYESNO) & ~MFINDINHELP, "M-X", 0, do_toggle, NO_HELP);
+	add_to_sclist(MMAIN, "M-C", 0, do_toggle, CONSTANT_SHOW);
+	add_to_sclist(MMAIN, "M-S", 0, do_toggle, SOFTWRAP);
+	add_to_sclist(MMAIN, "M-$", 0, do_toggle, SOFTWRAP);  /* Legacy keystroke. */
 #ifdef ENABLE_LINENUMBERS
-	add_to_sclist(MMAIN, "M-N", 0, do_toggle_void, LINE_NUMBERS);
-	add_to_sclist(MMAIN, "M-#", 0, do_toggle_void, LINE_NUMBERS);
+	add_to_sclist(MMAIN, "M-N", 0, do_toggle, LINE_NUMBERS);
+	add_to_sclist(MMAIN, "M-#", 0, do_toggle, LINE_NUMBERS);  /* Legacy keystroke. */
 #endif
-	add_to_sclist(MMAIN, "M-P", 0, do_toggle_void, WHITESPACE_DISPLAY);
+	add_to_sclist(MMAIN, "M-P", 0, do_toggle, WHITESPACE_DISPLAY);
 #ifdef ENABLE_COLOR
-	add_to_sclist(MMAIN, "M-Y", 0, do_toggle_void, NO_SYNTAX);
+	add_to_sclist(MMAIN, "M-Y", 0, do_toggle, NO_SYNTAX);
 #endif
 
-	/* Group of "Editing-behavior" toggles. */
-	add_to_sclist(MMAIN, "M-H", 0, do_toggle_void, SMART_HOME);
-	add_to_sclist(MMAIN, "M-I", 0, do_toggle_void, AUTOINDENT);
-	add_to_sclist(MMAIN, "M-K", 0, do_toggle_void, CUT_FROM_CURSOR);
+	/* Group of "Behavior" toggles. */
+	add_to_sclist(MMAIN, "M-H", 0, do_toggle, SMART_HOME);
+	add_to_sclist(MMAIN, "M-I", 0, do_toggle, AUTOINDENT);
+	add_to_sclist(MMAIN, "M-K", 0, do_toggle, CUT_FROM_CURSOR);
 #ifdef ENABLE_WRAPPING
-	add_to_sclist(MMAIN, "M-L", 0, do_toggle_void, BREAK_LONG_LINES);
+	add_to_sclist(MMAIN, "M-L", 0, do_toggle, BREAK_LONG_LINES);
 #endif
-	add_to_sclist(MMAIN, "M-O", 0, do_toggle_void, TABS_TO_SPACES);
-
-	/* Group of "Peripheral-feature" toggles. */
+	add_to_sclist(MMAIN, "M-O", 0, do_toggle, TABS_TO_SPACES);
 #ifdef ENABLE_MOUSE
-	add_to_sclist(MMAIN, "M-M", 0, do_toggle_void, USE_MOUSE);
+	add_to_sclist(MMAIN, "M-M", 0, do_toggle, USE_MOUSE);
 #endif
-	add_to_sclist(MMAIN, "M-Z", 0, do_toggle_void, SUSPENDABLE);
 #endif /* !NANO_TINY */
 
 	add_to_sclist(((MMOST & ~MMAIN) | MYESNO), "^C", 0, do_cancel, 0);
@@ -1378,28 +1407,26 @@ void shortcut_init(void)
 	add_to_sclist(MWHEREIS|MREPLACE, "^R", 0, flip_replace, 0);
 	add_to_sclist(MWHEREIS|MGOTOLINE, "^T", 0, flip_goto, 0);
 #ifdef ENABLE_HISTORIES
-	add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "^P", 0, get_history_older_void, 0);
-	add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "^N", 0, get_history_newer_void, 0);
+	add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "^P", 0, get_older_item, 0);
+	add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "^N", 0, get_newer_item, 0);
 #ifdef ENABLE_UTF8
 	if (using_utf8()) {
-		add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "\xE2\x96\xb2", KEY_UP, get_history_older_void, 0);
-		add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "\xE2\x96\xbc", KEY_DOWN, get_history_newer_void, 0);
+		add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "\xE2\x96\xb4", KEY_UP, get_older_item, 0);
+		add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "\xE2\x96\xbe", KEY_DOWN, get_newer_item, 0);
 	} else
 #endif
 	{
-		add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "Up", KEY_UP, get_history_older_void, 0);
-		add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "Down", KEY_DOWN, get_history_newer_void, 0);
+		add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "Up", KEY_UP, get_older_item, 0);
+		add_to_sclist(MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE|MFINDINHELP|MEXECUTE, "Down", KEY_DOWN, get_newer_item, 0);
 	}
 #endif
 #ifdef ENABLE_JUSTIFY
 	add_to_sclist(MGOTOLINE, "^W", 0, to_para_begin, 0);
 	add_to_sclist(MGOTOLINE, "^O", 0, to_para_end, 0);
 #endif
-	add_to_sclist(MGOTOLINE, "^Y", 0, to_first_line, 0);
-	add_to_sclist(MGOTOLINE, "^V", 0, to_last_line, 0);
 	/* Some people are used to having these keystrokes in the Search menu. */
-	add_to_sclist(MWHEREIS, "^Y", 0, to_first_line, 0);
-	add_to_sclist(MWHEREIS, "^V", 0, to_last_line, 0);
+	add_to_sclist(MGOTOLINE|MWHEREIS|MFINDINHELP, "^Y", 0, to_first_line, 0);
+	add_to_sclist(MGOTOLINE|MWHEREIS|MFINDINHELP, "^V", 0, to_last_line, 0);
 #ifdef ENABLE_BROWSER
 	add_to_sclist(MWHEREISFILE, "^Y", 0, to_first_file, 0);
 	add_to_sclist(MWHEREISFILE, "^V", 0, to_last_file, 0);
@@ -1409,20 +1436,21 @@ void shortcut_init(void)
 	add_to_sclist(MBROWSER, "End", KEY_END, to_last_file, 0);
 	add_to_sclist(MBROWSER, "^Home", CONTROL_HOME, to_first_file, 0);
 	add_to_sclist(MBROWSER, "^End", CONTROL_END, to_last_file, 0);
-	add_to_sclist(MBROWSER, "^_", 0, goto_dir, 0);
+	add_to_sclist(MBROWSER, SLASH_OR_DASH, 0, goto_dir, 0);
 	add_to_sclist(MBROWSER, "M-G", 0, goto_dir, 0);
+	add_to_sclist(MBROWSER, "^_", 0, goto_dir, 0);
 #endif
 	if (ISSET(SAVE_ON_EXIT) && !ISSET(PRESERVE))
 		add_to_sclist(MWRITEFILE, "^Q", 0, discard_buffer, 0);
 #ifndef NANO_TINY
-	add_to_sclist(MWRITEFILE, "M-D", 0, dos_format_void, 0);
-	add_to_sclist(MWRITEFILE, "M-M", 0, mac_format_void, 0);
+	add_to_sclist(MWRITEFILE, "M-D", 0, dos_format, 0);
+	add_to_sclist(MWRITEFILE, "M-M", 0, mac_format, 0);
 	/* Only when not in restricted mode, allow Appending, Prepending,
 	 * making backups, and executing a command. */
 	if (!ISSET(RESTRICTED) && !ISSET(VIEW_MODE)) {
-		add_to_sclist(MWRITEFILE, "M-A", 0, append_void, 0);
-		add_to_sclist(MWRITEFILE, "M-P", 0, prepend_void, 0);
-		add_to_sclist(MWRITEFILE, "M-B", 0, backup_file_void, 0);
+		add_to_sclist(MWRITEFILE, "M-A", 0, append_it, 0);
+		add_to_sclist(MWRITEFILE, "M-P", 0, prepend_it, 0);
+		add_to_sclist(MWRITEFILE, "M-B", 0, back_it_up, 0);
 		add_to_sclist(MINSERTFILE|MEXECUTE, "^X", 0, flip_execute, 0);
 	}
 	add_to_sclist(MINSERTFILE, "M-N", 0, flip_convert, 0);
@@ -1458,11 +1486,11 @@ void shortcut_init(void)
 #endif
 	add_to_sclist(MMOST & ~MFINDINHELP, "F1", KEY_F(1), do_help, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP, "F2", KEY_F(2), do_exit, 0);
-	add_to_sclist(MMAIN, "F3", KEY_F(3), do_writeout_void, 0);
+	add_to_sclist(MMAIN, "F3", KEY_F(3), do_writeout, 0);
 #ifdef ENABLE_JUSTIFY
-	add_to_sclist(MMAIN, "F4", KEY_F(4), do_justify_void, 0);
+	add_to_sclist(MMAIN, "F4", KEY_F(4), do_justify, 0);
 #endif
-	add_to_sclist(MMAIN, "F5", KEY_F(5), do_insertfile_void, 0);
+	add_to_sclist(MMAIN, "F5", KEY_F(5), do_insertfile, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP, "F6", KEY_F(6), do_search_forward, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "F7", KEY_F(7), do_page_up, 0);
 	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "F8", KEY_F(8), do_page_down, 0);
@@ -1472,30 +1500,33 @@ void shortcut_init(void)
 #ifdef ENABLE_SPELLER
 	add_to_sclist(MMAIN, "F12", KEY_F(12), do_spell, 0);
 #endif
-#if defined(KEY_CANCEL) && defined(KEY_SUSPEND) && defined(KEY_SIC)
-	add_to_sclist((MMOST & ~MMAIN) | MYESNO, "", KEY_CANCEL, do_cancel, 0);
-	add_to_sclist(MMAIN|MEXECUTE, "", KEY_SUSPEND, do_suspend_void, 0);
-	add_to_sclist(MMAIN, "", KEY_SIC, do_insertfile_void, 0);
-#endif
 #ifndef NANO_TINY
+	add_to_sclist((MMOST & ~MMAIN) | MYESNO, "", KEY_CANCEL, do_cancel, 0);
+	add_to_sclist(MMAIN, "", KEY_SIC, do_insertfile, 0);
 	/* Catch and ignore bracketed paste marker keys. */
 	add_to_sclist(MMOST|MBROWSER|MHELP|MYESNO, "", BRACKETED_PASTE_MARKER, do_nothing, 0);
+#else
+	add_to_sclist(MMOST|MBROWSER|MHELP|MYESNO, "", KEY_FRESH, full_refresh, 0);
 #endif
 }
 
 #ifndef NANO_TINY
 /* Return the textual description that corresponds to the given flag. */
-const char *flagtostr(int flag)
+const char *epithet_of_flag(int flag)
 {
 	switch (flag) {
-		case NO_HELP:
+		case ZERO:
 			/* TRANSLATORS: The next thirteen strings are toggle descriptions;
 			 * they are best kept shorter than 40 characters, but may be longer. */
+			return N_("Hidden interface");
+		case NO_HELP:
 			return N_("Help mode");
 		case CONSTANT_SHOW:
 			return N_("Constant cursor position display");
 		case SOFTWRAP:
 			return N_("Soft wrapping of overlong lines");
+		case LINE_NUMBERS:
+			return N_("Line numbering");
 		case WHITESPACE_DISPLAY:
 			return N_("Whitespace display");
 		case NO_SYNTAX:
@@ -1512,13 +1543,8 @@ const char *flagtostr(int flag)
 			return N_("Conversion of typed tabs to spaces");
 		case USE_MOUSE:
 			return N_("Mouse support");
-		case SUSPENDABLE:
-			return N_("Suspension");
-		case LINE_NUMBERS:
-			return N_("Line numbering");
 		default:
-			die("Bad toggle -- please report a bug\n");
-			return "";
+			return "Ehm...";
 	}
 }
 #endif /* !NANO_TINY */
